/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.Scanner;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.ws.AsyncHandler;
import javax.xml.ws.Response;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service1
 * reference gets called when service1 operation1 is invoked
 * 
 * The reference is declared with a Java interface that contains all the additional client side
 * asynchronous methods as defined by JAXWS
 */
@Service(Service1.class)
public class service1AsyncRefImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	
	// Property that controls how reference is invoked
	// "sync" = synchronous only
	// "asyncPoll" = asynchronous, polling
	// "asyncCallback" = asynchronous, callback
	// multiple of these strings separated by spaces cause the target service to be invoked multiple times,
	// each one with the specified style
	@Property(required=false)
	public String invokeMethods = "sync";
	
	// Property that controls the input parameter to the async service - it should be an array of strings that
	// matches in length the array of string in the invokeMethods property
	// "request" = a regular request
	// "exception" = cause the async service to throw an exception
	@Property(required=false)
	public String invokeParm = "request";
	
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public Service1WithAsyncMethods reference1 = null;
	
	private CallbackHandler callback = new CallbackHandler();
	private volatile String asyncResponse = null;
	private volatile CountDownLatch theLatch = null;
	
	private class CallbackHandler implements AsyncHandler<String> {

		public void handleResponse(Response<String> arg0) {
			try {
				asyncResponse = (String)arg0.get();
			} catch (ExecutionException e) {
				// An ExecutionException signals that the invocation of the target service threw the exception
				Throwable t = e.getCause();
				asyncResponse = "Exception received when Response.get() invoked: " + t.toString();
			} catch (Exception e) {
				asyncResponse = "Exception received when Response.get() invoked: " + e.getClass().toString() + " " + e.getMessage();
			} // end try
			theLatch.countDown();
		} // end handleResponse
		
	} // end class CallbackHandler

	public String operation1(String input) {
		String[] invokeArray = invokeMethods.split("\\s");
		Scanner inputs = new Scanner( invokeParm );
		
		String result = "";
		
		for( String invoke : invokeArray ) {
				
			if( invoke.equals("sync")) {
				result += reference1.operation1(inputs.next());
			} // end if
			
			if( invoke.equals("asyncPoll")) {
				// Invoke an async polling method...
				Response<String> theResponse = reference1.operation1Async( inputs.next() );
				if( theResponse == null ) {
					result += "No Result returned ";
				} else {
					result += "Result returned ";
				} // end if
				try {
					// Wait for the result to arrive
					result = theResponse.get(10, TimeUnit.SECONDS);
				} catch (ExecutionException e) {
					// An ExecutionException signals that the invocation of the target service threw the exception
					Throwable t = e.getCause();
					result += "Exception received when Response.get() invoked: " + t.toString();
				} catch (Exception e) {
					result += "Exception received when Response.get() invoked: " + e.getClass().toString() + " " + e.getMessage();
				} // end try
			} // end if
			
			if( invoke.equals("asyncCallback")) {
				// Create a new latch to coordinate between this invoking method and the callback method
				theLatch = new CountDownLatch(1);
				// Invoke async callback method...
				Future<?> theFuture = reference1.operation1Async(inputs.next(), callback);
				if( theFuture == null ) {
					result += "No Future returned ";
				} else {
					result += "Future returned ";
				} // end if
				// Wait for the asynchronous return of the response data from the invoked service operation
				try {
					theLatch.await(10, TimeUnit.SECONDS);
				} catch (InterruptedException e) {
					result += "Interrupted exception received when waiting for the async callback";
				} // end try
				result += asyncResponse;
			} // end if
		
		} // end for
		
		return serviceName + " operation1 invoked" + " " + result;
	} // end method operation1

}
