/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java STATELESS component implementation which tests the overall Lifecycle sequence of an SCA
 * Java implementation class
 * 1 service with interface Service1
 * 1 reference (1..1) with interface Service1
 *
 */
@Service(Service1.class)
public class service1StatelessLifecycleImpl implements Service1 {
	
	private enum Lifecycle { Constructing, Injecting, InjectingRefs, Initializing, Running, Destroying, Terminated }
	private volatile Lifecycle lifecycleState;
	
	// Accumulates the series of actions that take place
	private String actionSequence = "";
	
	@Property
	public String serviceName = "service1";
	@Reference
	public DataStore dataStore = null;
	
	@Property
	public String property3 = null;
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public Service1 reference3 = null;
	
	private Service1 reference1 	= null;
	private Service1 reference2 	= null;
	private String property1 		= null;
	private String property2 		= null;
	
	/**
	 * Constructor - with Reference and Property parameters for constructor injection
	 * @param property1
	 * @param reference1
	 */
	@Constructor
	public service1StatelessLifecycleImpl( @Property(name="property1") String property1Parm, 
			                               @Reference(name="reference1") Service1 reference1Parm ) {
		lifecycleState = Lifecycle.Constructing;
		actionSequence = "Constructing";
		
		if( property1Parm != null ) actionSequence += " property1 injected";
		if( reference1Parm != null ) actionSequence += " reference1 injected";
		property1 	= property1Parm;
		reference1	= reference1Parm;
		
		// Transition to injecting state
		lifecycleState = Lifecycle.Injecting;
	} // end service1StatelessLifecycleImpl constructor
	
	@Property(required=true)
	public void setproperty2( String property2Parm ){
		if(property2Parm == null) return;
		property2 = property2Parm;
				
		actionSequence += " property2 injected";
		lifecycleState = Lifecycle.InjectingRefs;
	} // end setProperty2
	
	/**
	 * Setter for reference2 reference
	 */
	@Reference(required=true)
	public void setreference2( Service1 reference2Parm ) {
		if(reference2Parm == null) return;
		reference2 = reference2Parm;
		
		actionSequence += " reference2 injected";
		lifecycleState = Lifecycle.Initializing;
	} // end setReference2
	
	@Init
	public void initialize() {
		actionSequence += " Init invoked";
		
		// First check that ALL injection is complete
		if( (property1 == null) ||
			(property2 == null) ||
			(property3 == null) ||
			(reference1 == null) ||
			(reference2 == null) ||
			(reference3 == null) ) {
			actionSequence += " 1 or more Property or Reference is not set";
			return;
		} // end if
		
		actionSequence += " Init completed";
		lifecycleState = Lifecycle.Running;
	} // end method initialize

	public String operation1(String input) {
		String result;
		if( lifecycleState != Lifecycle.Running ) {
			result = serviceName + " operation1 invoked out of lifecycle sequence";
		} else { 
			String result1 = reference1.operation1("1");
			String result2 = reference2.operation1("2");
			String result3 = reference3.operation1("3");
			result = serviceName + " operation1 invoked" +
			         " " + result1 + " " + result2 + " " + result3 +
	                 " " + property1 + " " + property2 + " " + property3;	
		} // end if

		actionSequence += " ";
		actionSequence += result;
		
		lifecycleState = Lifecycle.Destroying;  // This is stateless - only get invoked once!
		return result;
	} // end operation1
	
	@Destroy
	public void destroying() {
		actionSequence += " Destroy invoked";
		
		// Store the final action sequence data...
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.Terminated;
	} // end method destroying

}
