/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 3 references 
 * - requiredRef (1..1) with interface Service1
 * - singledRef (0..1) with interface Service1 (intended to be unwired)
 * - multiRef (0..n) with interface Service1
 * 
 * This implementation performs invocations of the getServiceReference() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCgetServiceReferenceImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";
	@Reference
	public Service1 requiredRef;		// 1..1 multiplicity
	@Reference(required=false)
	public Service1 singleRef;			// 0..1 multiplicity - *SHOULD NOT BE WIRED*
	@Reference(required=false)
	public Service1[] multiRef;			// 0..n multiplicity

	public Service1CCgetServiceReferenceImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked getServiceReference";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9070 - IllegalArgumentException thrown if called for an x..n reference
		try {
		    componentContext.getServiceReference(Service1.class, "multiRef");
		    responseAccumulator += " did not get IllegalArgumentException for 0..n ref";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for 0..n ref";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for 0..n ref: " + e.toString();
		} // end try
		
		// JCA-TA-9006 - IllegalArgumentException thrown if named reference does not have the businessInterface specified
		try {
		    componentContext.getServiceReference(Service2.class, "requiredRef");
		    responseAccumulator += " did not get IllegalArgumentException for invalid businessInterface";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid businessInterface";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid businessInterface: " + e.toString();
		} // end try	
		
		// JCA-TA-9007 - IllegalArgumentException thrown if component does not have the named reference
		try {
		    componentContext.getServiceReference(Service1.class, "randomRef");
		    responseAccumulator += " did not get IllegalArgumentException invalid reference name";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid reference name";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid reference name: " + e.toString();
		} // end try
		
		// JCA-TA-9008 - returns null for a 0..1 reference that is not wired
		try {
		    ServiceReference<Service1> serviceRef = componentContext.getServiceReference(Service1.class, "singleRef");
		    if( serviceRef == null ) {
			    responseAccumulator += " expected null ServiceReference for unwired 0..1 reference";
		    } else {
		    	responseAccumulator += " unexpected non-null ServiceReference for unwired 0..1 reference";
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for unwired 0..1 reference";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for unwired 0..1 reference: " + e.toString();
		} // end try
		
		// JCA-TA-9014 - returns a valid ServiceReference for a valid reference which is wired to a target service
		try {
		    ServiceReference<Service1> serviceRef = componentContext.getServiceReference(Service1.class, "requiredRef");
		    if( serviceRef == null ) {
			    responseAccumulator += " unexpected null ServiceReference for wired 1..1 reference";
		    } else {
		    	responseAccumulator += " expected non-null ServiceReference for wired 1..1 reference";
		    	// Actually invoke the target service to verify that the returned ServiceReference is indeed a proxy for that service
		    	Service1 targetService = serviceRef.getService();
		    	responseAccumulator += " " + targetService.operation1( "dynamic" );
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for wired 1..1 reference";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for wired 1..1 reference: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

} // end class
