/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.ServiceUnavailableException;
import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java STATELESS component implementation which tests the overall Lifecycle sequence of an SCA
 * Java implementation class
 * 1 service with interface Service1
 * 1 reference (1..1) with interface Service1
 *
 */
@Service(Service1.class)
public class service1LifecycleExceptionsImpl implements Service1 {
	
	private enum Lifecycle { Constructing, Injecting, InjectingRefs, Initializing, Running, Destroying, Terminated }
	private volatile Lifecycle lifecycleState;
	
	// Accumulates the series of actions that take place
	private String actionSequence = "";
	
	private String serviceName = "service1";
	private DataStore dataStore = null;
	private Service1 uninitRef = null;
	
	@Property
	public String property3 = null;
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public Service1 reference3 = null;
	
	private Service1 reference1 	= null;
	private Service1 reference2 	= null;
	private String property1 		= null;
	private String property2 		= null;
	
	private String exceptionPoint	= "constructing";
	
	/**
	 * Constructor - with Reference and Property parameters for constructor injection
	 * @param property1Parm - a String property
	 * @param exceptionParm - a String property indicating the point at which to throw an exception
	 * @param serviceNameParm - a String property containing the name of this service component
	 * @param reference1 - a Reference to a service with interface Service1
	 * @param dataStoreParm - a Reference to a service with interface DataStore
	 * @param uninitRef - a Reference to a service with interface Service1 that is uninitialized
	 * @throws Exception - thrown if the exceptionParm == "constructing"
	 */
	@Constructor
	public service1LifecycleExceptionsImpl( @Property(name="property1") String property1Parm,
			                                @Property(name="exceptionPoint") String exceptionParm,
			                                @Property(name="serviceName") String serviceNameParm,
			                                @Reference(name="reference1") Service1 reference1Parm,
			                                @Reference(name="dataStore") DataStore dataStoreParm,
			                                @Reference(name="uninitRef") Service1 uninitRefParm ) throws Exception {
		if( exceptionParm != null ) exceptionPoint = exceptionParm;
		if( serviceNameParm != null ) serviceName = serviceNameParm;
		if( uninitRefParm != null ) uninitRef = uninitRefParm;
		dataStore = dataStoreParm;
		
		lifecycleState = Lifecycle.Constructing;
		actionSequence = "Constructing";
		
		if( "constructing".equals(exceptionPoint) ) {
			actionSequence += " exception thrown";
			dataStore.storeData( serviceName, actionSequence );
			throw new Exception("Exception thrown during constructor by service1LifecycleExceptionsImpl");
		} // end if
		
		if( property1Parm != null ) actionSequence += " property1 injected";
		if( reference1Parm != null ) actionSequence += " reference1 injected";
		property1 	= property1Parm;
		reference1	= reference1Parm;
		
		dataStore.storeData( serviceName, actionSequence );
		// Transition to injecting state
		lifecycleState = Lifecycle.Injecting;
	} // end service1StatelessLifecycleImpl constructor
	
	/**
	 * Setter for property2 property
	 * @param property2Parm - property value
	 * @throws Exception
	 */
	@Property(required=true)
	public void setproperty2( String property2Parm ) throws Exception {
		if(property2Parm == null) return;
		property2 = property2Parm;
		actionSequence += " property2 injected";

		if( "propertyinjection".equals(exceptionPoint) ) {
			actionSequence += " exception thrown";
			dataStore.storeData( serviceName, actionSequence );
			throw new Exception("Exception thrown during property injection by service1LifecycleExceptionsImpl");
		} // end if
				
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.InjectingRefs;
	} // end setProperty2
	
	/**
	 * Setter for reference2 reference
	 */
	@Reference(required=true)
	public void setreference2( Service1 reference2Parm ) throws Exception {
		if(reference2Parm == null) return;
		reference2 = reference2Parm;
		actionSequence += " reference2 injected";
		
		if( "referenceinjection".equals(exceptionPoint) ) {
			actionSequence += " exception thrown";
			dataStore.storeData( serviceName, actionSequence );
			throw new Exception("Exception thrown during reference injection by service1LifecycleExceptionsImpl");
		} // end if
		
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.Initializing;
	} // end setReference2
	
	@Init
	public void initialize() throws Exception {
		actionSequence += " Init invoked";
		
		// First check that ALL injection is complete
		if( (property1 == null) ||
			(property2 == null) ||
			(property3 == null) ||
			(reference1 == null) ||
			(reference2 == null) ||
			(reference3 == null) ) {
			actionSequence += " 1 or more Property or Reference is not set";
			dataStore.storeData( serviceName, actionSequence );
			return;
		} // end if
		
		if( "initmethod".equals(exceptionPoint) ) {
			actionSequence += " exception thrown";
			dataStore.storeData( serviceName, actionSequence );
			throw new Exception("Exception thrown during @Init method by service1LifecycleExceptionsImpl");
		} // end if
		
		if( "calluninit".equals(exceptionPoint) ) {
			try {
				actionSequence += " calling uninitialized service";
				String result = uninitRef.operation1("called");
				actionSequence += " no exception received";
			} catch (Exception e) {
				if( e instanceof ServiceUnavailableException ) {
					actionSequence += " ServiceUnavailable exception received";
				} else {
					actionSequence += " other exception received";
					// e.printStackTrace();
				} // end if 
			} // end try
		} // end if
		
		actionSequence += " Init completed";
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.Running;
	} // end method initialize

	public String operation1(String input) {
		String result;
		if( lifecycleState != Lifecycle.Running ) {
			result = serviceName + " operation1 invoked out of lifecycle sequence";
		} else { 
			String result1 = reference1.operation1("1");
			String result2 = reference2.operation1("2");
			String result3 = reference3.operation1("3");
			result = serviceName + " operation1 invoked" +
			         " " + result1 + " " + result2 + " " + result3 +
	                 " " + property1 + " " + property2 + " " + property3;	
		} // end if

		actionSequence += " ";
		actionSequence += result;
		
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.Destroying;  // This is stateless - only get invoked once!
		return result;
	} // end operation1
	
	@Destroy
	public void destroying() throws Exception {
		actionSequence += " Destroy invoked";
		
		if( "destroying".equals(exceptionPoint) ) {
			actionSequence += " exception thrown";
			dataStore.storeData( serviceName, actionSequence );
			throw new Exception("Exception thrown during @Destroy method by service1LifecycleExceptionsImpl");
		} // end if
		
		// Store the final action sequence data...
		dataStore.storeData( serviceName, actionSequence );
		lifecycleState = Lifecycle.Terminated;
	} // end method destroying

} // end class service1LifecycleExceptionsImpl
